@@ -10,6 +10,10 @@ import pandas as pd
 import numpy as np
 import warnings
 import logging
+from collections import defaultdict
+from numba.core.errors import NumbaDeprecationWarning, NumbaPendingDeprecationWarning
+warnings.filterwarnings('ignore', category=NumbaDeprecationWarning)
+warnings.filterwarnings('ignore', category=NumbaPendingDeprecationWarning)
 import shap
 from shap.plots._beeswarm import pl
 
@@ -33,6 +37,7 @@ class Shap:
         # the list of columns that will be created after sorting the shap values. These are then used for creating the
         # narratives
         self.value_feature_col_list = self.get_value_feature_col_list()
+        self.final_filenames = defaultdict(list)
 
     @property
     def conf(self):
@@ -66,13 +71,14 @@ class Shap:
         return value_feature_col_list
 
     @timeit("run shap analysis")
-    def run_shap(self, test_data, tag):
+    def run_shap(self, test_data, tag, index='', seg_id=''):
         """The main driver method for performing shap analysis calculations. This is the only method that is usually
         called after instantiating the Shap object.
 
         :param test_data: the test dataset. This data set contains the training data as well as the id columns.
         :param tag: the model tag. This is being used to name the files.
-        is not being used.
+        :param index: if we only loaded test data in chunk, this will be the chunk index
+        :param seg_id: the id of the segment. Only applicable when using engines with segmentation functionality.
         :return: None
         """
         try:
@@ -88,7 +94,7 @@ class Shap:
         # we handle the plots here because in some situations we need only on item in shap_values list and in
         # some situations we need multiple items. Also, we need to plot things before converting shap_values into
         # numpy arrays
-        self.handle_plots(tag, shap_values, test_data)
+        self.handle_plots(tag, shap_values, test_data, seg_id=seg_id)
         shap_values = np.array(shap_values)
 
         # we need the means to use them for storytelling
@@ -99,17 +105,18 @@ class Shap:
                          "each output")
             for i, class_name in enumerate(self.models_dict_item["model"].classes_):
                 logging.info(f"Processing shap values for class: {class_name}")
-                self.process_shap_data(tag, shap_values[i], test_data, class_name=class_name)
+                self.process_shap_data(tag, shap_values[i], test_data, class_name=class_name, index=index, seg_id=seg_id)
         else:
-            self.process_shap_data(tag, shap_values, test_data)
+            self.process_shap_data(tag, shap_values, test_data, class_name=None, index=index, seg_id=seg_id)
 
-    def handle_plots(self, tag, shap_values, test_data):
+    def handle_plots(self, tag, shap_values, test_data, seg_id=''):
         """The main method for using shap library and plotting the data. This method checks if plotting functionality
         was activated or not. If it was activated, it continues with the rest of operations.
 
         :param tag: the model tag. This is being used to name the files.
         :param shap_values: shap values with the same format that comes out of shap library
         :param test_data: the test dataframe
+        :param seg_id: the id of the segment. Only applicable when using engines with segmentation functionality.
         :return: None
         """
         if not self.conf.analysis.shap_plot_activate:
@@ -128,12 +135,18 @@ class Shap:
         for plot_type in ["bar", "dot"]:
             logging.info("Making {plot_type} plot for shap values...".format(plot_type=plot_type))
 
+            if seg_id == '':
+                path = self._conf.model.models_dict[tag]["path"]
+            else:
+                path = self._conf.model.models_dict[tag][seg_id]["path"]
             filename = SHAP_PLOT.format(
-                path=self._conf.model.models_dict[tag]["path"],
+                path=path,
                 output_name=self.conf.workspace.name,
                 tag=str(tag),
                 type=plot_type,
-                sep_timetag=str(self.conf.sep_timetag))
+                sep_timetag=str(self.conf.sep_timetag),
+                seg_id=f"_{seg_id}" if seg_id != '' else seg_id
+            )
             try:
                 # this can fail for dot plot and multi-class classifications problem. In that case, we still want to make
                 # the plots but we need to send the data for each class separately.
@@ -149,12 +162,14 @@ class Shap:
                 for i, class_name in enumerate(self.models_dict_item["model"].classes_):
                     logging.info("Plotting shap values for class: {c}".format(c=class_name))
                     filename = SHAP_PLOT_ClASS.format(
-                        path=self._conf.model.models_dict[tag]["path"],
+                        path=path,
                         output_name=self.conf.workspace.name,
                         tag=str(tag),
                         type=plot_type,
                         class_name=class_name,
-                        sep_timetag=str(self.conf.sep_timetag))
+                        sep_timetag=str(self.conf.sep_timetag),
+                        seg_id=f"_{seg_id}" if seg_id != '' else seg_id
+                    )
                     shap.summary_plot(shap_values[i],
                                       test_data[self.models_dict_item["train_features"]],
                                       show=False,
@@ -166,7 +181,7 @@ class Shap:
                     pl.clf()
 
     @timeit("process the shap data")
-    def process_shap_data(self, tag, shap_values, test_data, class_name=None):
+    def process_shap_data(self, tag, shap_values, test_data, class_name=None, index='', seg_id=''):
         """The main method for processing the shap data. This include converting the shap_values numpy arrays in pandas
         dataframe, calling the narrative method to sort the shap values and create the narratives, and saving the final
         data to disk (by calling the write_shap method)
@@ -175,6 +190,8 @@ class Shap:
         :param shap_values: a numpy array that contain shap values
         :param test_data: the test dataframe
         :param class_name: the label of the class for classification problems
+        :param index: if we only loaded test data in chunk, this will be the chunk index
+        :param seg_id: the id of the segment. Only applicable when using engines with segmentation functionality.
         :return: None
         """
         logging.info("Processing the shap data...")
@@ -195,7 +212,7 @@ class Shap:
             shap_df, test_data = parrallelize_dataframes(shap_df, test_data, self.handle_narrative, self.conf.analysis.shap_num_cores)
             # this is the serial code
             # self.create_narrative(shap_df, test_data[self.models_dict_item["train_features"]], class_name=class_name if class_name is not None else "target")
-        self.write_shap(tag, shap_df, test_data[self.conf.column.id_cols], class_name="_{c}".format(c=class_name) if class_name is not None else '')
+        self.write_shap(tag, shap_df, test_data[self.conf.column.id_cols], class_name="_{c}".format(c=class_name) if class_name is not None else '', index=index, seg_id=seg_id)
 
     @timeit("compute the mean of the columns")
     def calculate_the_mean(self, data, cols):
@@ -440,7 +457,7 @@ class Shap:
         return " ".join(narrative)
 
     @timeit("write the shap output")
-    def write_shap(self, tag, shap_df, data, class_name=''):
+    def write_shap(self, tag, shap_df, data, class_name='', index='', seg_id=''):
         """Write the shap dataframe along with any additional columns (typically id columns) to disk. This method also
         makes a call to drop_cols method of the DataProcessor class to drop the unsorted column if not requested.
 
@@ -448,6 +465,8 @@ class Shap:
         :param shap_df: the shap dataframe
         :param data: the additional data to be concatenated with the shap dataframe. This is typically id columns
         :param class_name: the class name in case of multi-class classification problem.
+        :param index: if we only loaded test data in chunk, this will be the chunk index
+        :param seg_id: the id of the segment. Only applicable when using engines with segmentation functionality.
         :return: None
         """
         logging.info("Writing the shap output data...")
@@ -461,12 +480,30 @@ class Shap:
         shap_df.reset_index(drop=True, inplace=True)
         results = pd.concat([data, shap_df], axis=1, ignore_index=False)
 
+        if seg_id == '':
+            directory_path = self._conf.model.models_dict[tag]["path"]
+        else:
+            directory_path = self._conf.model.models_dict[tag][seg_id]["path"]
+        final_filename = SHAP_PATH.format(
+            output_name=self._conf.workspace.name,
+            tag=str(tag),
+            class_name=class_name,
+            sep_timetag=str(self._conf.sep_timetag),
+            index='',
+            seg_id=''
+        )
+
         filename = SHAP_PATH.format(
             output_name=self._conf.workspace.name,
             tag=str(tag),
             class_name=class_name,
-            sep_timetag=str(self._conf.sep_timetag))
+            sep_timetag=str(self._conf.sep_timetag),
+            index=index,
+            seg_id=f"_{seg_id}" if seg_id != '' else seg_id
+        )
+
+        self.final_filenames[directory_path + final_filename].append(directory_path + filename)
 
         output = OutputHandler(self._conf)
-        output.save_file(self._conf.model.models_dict[tag]["path"], filename, results, add_timetag=False)
+        output.save_file(directory_path, filename, results, add_timetag=False)
         logging.info("Successfully wrote the shap output data")
