@@ -13,7 +13,9 @@ from learner.configuration.workspace import WorkspaceConfiguration
 from learner.configuration.data import DataConfiguration
 from learner.configuration.column import ColumnConfiguration
 from learner.validator.input_validator import validate_subset_list, validate_intersection_cols
-from learner.configuration.supported_items import SUPPORTED_NARRATIVE_METHODS, SUPPORTED_ENGINE_FOR_PREDICTIONS_VS_ACTUALS_PLOT
+from learner.configuration.supported_items import (SUPPORTED_NARRATIVE_METHODS,
+                                                   SUPPORTED_ENGINE_FOR_PREDICTIONS_VS_ACTUALS_PLOT,
+                                                   SUPPORTED_ENGINE_FOR_CALIBRATION_CURVE_PLOT)
 
 
 class AnalysisConfiguration:
@@ -62,7 +64,7 @@ class AnalysisConfiguration:
         self.shap_num_cores = get_value(self._json_config, None, "analysis", "shap_params", "num_cores")
         self.shap_narrative_activate = get_value(self._json_config, False, "analysis", "shap_params", "narrative_params", "activate")
         self.shap_include_raw_values = get_value(self._json_config, False, "analysis", "shap_params", "include_raw_values")
-        self.shap_plot_activate = get_value(self._json_config, False, "analysis", "shap_params", "plot_params", "activate")
+        self.shap_plot_activate = self.get_shap_plot_activate()
         self.shap_include_sorted_values = self.get_shap_include_sorted_values()
         self.shap_exclude_cols = self.get_shap_exclude_cols()
         self.shap_use_training_mean = get_value(self._json_config, False, "analysis", "shap_params", "use_training_mean")
@@ -81,6 +83,21 @@ class AnalysisConfiguration:
         self.predictions_vs_actuals_symbol_size = get_value(self._json_config, 1, "analysis", "predictions_vs_actuals_params", "symbol_size")
         self.predictions_vs_actuals_symbol_color = get_value(self._json_config, "blue", "analysis", "predictions_vs_actuals_params", "symbol_color")
 
+        self.calibration_curve_activate = self.get_calibration_curve_activate()
+        self.calibration_n_bins = get_value(self._json_config, 10, "analysis", "calibration_curve_params", "n_bins")
+        self.calibration_strategy = get_value(self._json_config, "uniform", "analysis", "calibration_curve_params", "strategy")
+        self.calibration_plot_line = get_value(self._json_config, False, "analysis", "calibration_curve_params", "plot_line")
+        self.calibration_curve_width = get_value(self._json_config, 10, "analysis", "calibration_curve_params", "width")
+        self.calibration_curve_height = get_value(self._json_config, 10, "analysis", "calibration_curve_params", "height")
+        self.calibration_curve_x_labelsize = get_value(self._json_config, 10, "analysis", "calibration_curve_params", "x_labelsize")
+        self.calibration_curve_y_labelsize = get_value(self._json_config, 10, "analysis", "calibration_curve_params", "y_labelsize")
+        self.calibration_curve_x_label = get_value(self._json_config, "Mean predicted probability", "analysis", "calibration_curve_params", "x_label")
+        self.calibration_curve_y_label = get_value(self._json_config, "Fraction of positives", "analysis", "calibration_curve_params", "y_label")
+        self.calibration_curve_xy_min = get_value(self._json_config, -0.02, "analysis", "calibration_curve_params", "xy_min")
+        self.calibration_curve_xy_max = get_value(self._json_config, 1.02, "analysis", "calibration_curve_params", "xy_max")
+        self.calibration_curve_symbol_size = get_value(self._json_config, 10, "analysis", "calibration_curve_params", "symbol_size")
+        self.calibration_curve_symbol_color = get_value(self._json_config, "blue", "analysis", "calibration_curve_params", "symbol_color")
+
     @property
     def json_config(self):  # pragma: no cover
         return self._json_config
@@ -110,10 +127,16 @@ class AnalysisConfiguration:
 
     def get_shap_activate(self):
         try:
-            activate = self._json_config["analysis"]["shap_params"]["activate"]
+            return self._json_config["analysis"]["shap_params"]["activate"]
+        except KeyError:
+            return False
+
+    def get_shap_plot_activate(self):
+        try:
+            activate = self._json_config["analysis"]["shap_params"]["plot_params"]["activate"]
             if activate:
-                warnings.warn("shap analysis was activated. Learner will attempt to load the entire test dataset. "
-                              "Please ensure you have enough resources available", Warning)
+                warnings.warn("Plotting the shap values was activated. Learner will attempt to load the entire "
+                              "test dataset. Please ensure you have enough resources available", Warning)
                 # setting the chunksize to a big number so that the entire test data will be loaded
                 self._data.test_chunksize = 9000000000
             return activate
@@ -251,3 +274,20 @@ class AnalysisConfiguration:
             return activate
         except KeyError:
             return False
+
+    def get_calibration_curve_activate(self):
+        try:
+            activate = self._json_config["analysis"]["calibration_curve_params"]["activate"]
+            if activate:
+                validate_subset_list(parent_list=SUPPORTED_ENGINE_FOR_CALIBRATION_CURVE_PLOT,
+                                     parent_name="acceptable engines for plotting calibration curves",
+                                     subset_list=[self._engine],
+                                     subset_name="defined engine")
+                if not self._data.validation_score_activate:
+                    logging.critical("scoring_params in validation_params was not activated. Learner will not make "
+                                     "predictions and therefore cannot plot the calibration curves. Please update "
+                                     "your configuration file and try again. Exiting...")
+                    sys.exit(1)
+            return activate
+        except KeyError:
+            return False
