@@ -24,11 +24,12 @@ from learner.utilities.templates import TRAIN_VALIDATION_SPLIT
 class DataConfiguration:
     """Parse all input variables in the file section including the location of the train/test files, nrows, etc."""
 
-    def __init__(self, json_config, workspace: WorkspaceConfiguration, sample: SampleConfiguration, engine, sep_timetag):
+    def __init__(self, json_config, workspace: WorkspaceConfiguration, sample: SampleConfiguration, engine, sep_timetag, dml):
         self._json_config = json_config
         self._workspace = workspace
         self._sample = sample
         self._engine = engine
+        self._dml = dml
         self.sep_timetag = sep_timetag
 
         self.train_format = self.get_train_format()
@@ -111,6 +112,20 @@ class DataConfiguration:
         self.validation_column_name = get_value(self._json_config, "prediction", "data", "validation_params", "scoring_params", "column_name")
         self.validation_join_cols = self.get_validation_join_cols()
 
+        self.model_y_validation_prediction_type = None
+        self.model_y_validation_score_activate = self.get_model_y_validation_score_activate()
+        self.model_y_validation_score_types = self.get_model_y_validation_score_types()
+        self.model_y_validation_add_timetag = get_value(self._json_config, True, "data", "validation_params", "model_y_scoring_params", "add_timetag")
+        self.model_y_validation_column_name = get_value(self._json_config, "prediction", "data", "validation_params", "model_y_scoring_params", "column_name")
+        self.model_y_validation_join_cols = self.get_model_y_validation_join_cols()
+
+        self.model_t_validation_prediction_type = None
+        self.model_t_validation_score_activate = self.get_model_t_validation_score_activate()
+        self.model_t_validation_score_types = self.get_model_t_validation_score_types()
+        self.model_t_validation_add_timetag = get_value(self._json_config, True, "data", "validation_params", "model_t_scoring_params", "add_timetag")
+        self.model_t_validation_column_name = get_value(self._json_config, "prediction", "data", "validation_params", "model_t_scoring_params", "column_name")
+        self.model_t_validation_join_cols = self.get_model_t_validation_join_cols()
+
         self.test_prediction_activate = self.get_test_prediction_activate()
         self.test_prediction_type = self.get_test_prediction_type()
         self.test_add_timetag = get_value(self._json_config, True, "data", "test_params", "prediction_params", "add_timetag")
@@ -532,12 +547,17 @@ class DataConfiguration:
                         sys.exit(1)
         except KeyError:
             if self.validation_score_activate:
-                warnings.warn("scoring_params was activated but no score types was defined.", UserWarning)
-                self.validation_score_activate = False
+                if self._engine != "DML":
+                    warnings.warn("scoring_params was activated but no score types was defined.", UserWarning)
+                    self.validation_score_activate = False
+                else:
+                    return {"MSE": {}}
             return score_types_dict
 
-        self.validate_score_types(score_types_dict)
-        self.update_validation_prediction_type(score_types_dict)
+        self.validate_score_types(score_types_dict, self.validation_score_activate, self._engine)
+        self.validation_prediction_type = self.update_validation_prediction_type(score_types_dict,
+                                                                                 self._engine,
+                                                                                 self.validation_score_activate)
         return score_types_dict
 
     def get_validation_join_cols(self):
@@ -547,15 +567,17 @@ class DataConfiguration:
             join_cols = []
         return join_cols
 
-    def validate_score_types(self, score_types_dict):
-        if self.validation_score_activate:
+    def validate_score_types(self, score_types_dict, activate, engine, display_engine=None):
+        if display_engine is None:
+            display_engine = engine
+        if activate is True:
             for score_type, params in score_types_dict.items():
-                if score_type not in SUPPORTED_SCORE_TYPES[self._engine]:
+                if score_type not in SUPPORTED_SCORE_TYPES[engine]:
                     if "file" in params:
                         self._get_score_function(score_type, params)
                     else:
                         logging.error("Invalid types in 'scoring_params', supported score types for %s engine "
-                                      "are %s. Exiting...", self._engine, sorted(SUPPORTED_SCORE_TYPES[self._engine]))
+                                      "are %s. Exiting...", display_engine, sorted(SUPPORTED_SCORE_TYPES[engine]))
                         sys.exit(1)
 
     def _get_score_function(self, score_type, params):
@@ -569,16 +591,125 @@ class DataConfiguration:
                              f"The error is {e}. Exiting...")
             sys.exit(1)
 
-    def update_validation_prediction_type(self, score_types_dict):
+    def update_validation_prediction_type(self, score_types_dict, engine, activate):
         required_pred_type = set()
-        if self.validation_score_activate:
+        prediction_type = None
+        if activate:
             for score_type, params in score_types_dict.items():
                 if "file" not in params:
-                    required_pred_type.add(SUPPORTED_SCORE_TYPES[self._engine][score_type])
+                    required_pred_type.add(SUPPORTED_SCORE_TYPES[engine][score_type])
                     if len(required_pred_type) > 1:
-                        self.validation_prediction_type = "all"
+                        prediction_type = "all"
                     else:
-                        self.validation_prediction_type = list(required_pred_type)[0]
+                        prediction_type = list(required_pred_type)[0]
+            return prediction_type
+
+    def get_model_y_validation_score_activate(self):
+        try:
+            score = self._json_config["data"]["validation_params"]["model_y_scoring_params"]["activate"]
+            if score and not self.sample.split_activate and not self.validation_query_activate and not self.validation_location:
+                logging.error("It was requested to score the validation data but no query or location is provided "
+                              "for the validation data, and train data is not being split. Please update the "
+                              "configuration file and try again. Exiting...")
+                sys.exit(1)
+            return score
+        except KeyError:
+            return False
+
+    def get_model_y_validation_score_types(self):
+        score_types_dict = {}
+        try:
+            score_types_list = self._json_config["data"]["validation_params"]["model_y_scoring_params"]["types"]
+            if score_types_list:
+                if self.model_y_validation_score_activate is False:
+                    warnings.warn("score types is not empty and model_y_scoring_params is not activated.", UserWarning)
+                    return {}
+                for score_type in score_types_list:
+                    if isinstance(score_type, dict):
+                        try:
+                            score_types_dict[score_type["type"]] = score_type["params"]
+                        except KeyError:
+                            logging.exception("When passing a dictionary as an item to score_types, both type and "
+                                              "params key must be present. The value of params can be a empty "
+                                              "dictionary. Scoring type can also be a single string if you there "
+                                              "is no extra parameters for the score type Exiting...")
+                            sys.exit(1)
+                    elif isinstance(score_type, str):
+                        score_types_dict[score_type] = {}
+                    else:
+                        logging.error("Elements in score types must be either a string or a dictionary. "
+                                      "Exiting...")
+                        sys.exit(1)
+        except KeyError:
+            if self.model_y_validation_score_activate:
+                warnings.warn("model_y_scoring_params was activated but no score types was defined.", UserWarning)
+                self.validation_score_activate = False
+            return score_types_dict
+
+        self.validate_score_types(score_types_dict, self.model_y_validation_score_activate, 'Regressor', self._engine)
+        self.model_y_validation_prediction_type = self.update_validation_prediction_type(score_types_dict, 'Regressor', self.model_y_validation_score_activate)
+        return score_types_dict
+
+    def get_model_y_validation_join_cols(self):
+        try:
+            join_cols = self._json_config["data"]["validation_params"]["model_y_scoring_params"]["join_cols"]
+        except KeyError:
+            join_cols = []
+        return join_cols
+
+    def get_model_t_validation_score_activate(self):
+        try:
+            score = self._json_config["data"]["validation_params"]["model_t_scoring_params"]["activate"]
+            if score and not self.sample.split_activate and not self.validation_query_activate and not self.validation_location:
+                logging.error("It was requested to score the validation data but no query or location is provided "
+                              "for the validation data, and train data is not being split. Please update the "
+                              "configuration file and try again. Exiting...")
+                sys.exit(1)
+            return score
+        except KeyError:
+            return False
+
+    def get_model_t_validation_score_types(self):
+        score_types_dict = {}
+        try:
+            score_types_list = self._json_config["data"]["validation_params"]["model_t_scoring_params"]["types"]
+            if score_types_list:
+                if self.model_t_validation_score_activate is False:
+                    warnings.warn("score types is not empty and model_t_scoring_params is not activated.", UserWarning)
+                    return {}
+                for score_type in score_types_list:
+                    if isinstance(score_type, dict):
+                        try:
+                            score_types_dict[score_type["type"]] = score_type["params"]
+                        except KeyError:
+                            logging.exception("When passing a dictionary as an item to score_types, both type and "
+                                              "params key must be present. The value of params can be a empty "
+                                              "dictionary. Scoring type can also be a single string if you there "
+                                              "is no extra parameters for the score type Exiting...")
+                            sys.exit(1)
+                    elif isinstance(score_type, str):
+                        score_types_dict[score_type] = {}
+                    else:
+                        logging.error("Elements in score types must be either a string or a dictionary. "
+                                      "Exiting...")
+                        sys.exit(1)
+        except KeyError:
+            if self.model_t_validation_score_activate:
+                warnings.warn("model_t_scoring_params was activated but no score types was defined.", UserWarning)
+                self.validation_score_activate = False
+            return score_types_dict
+
+        temp_engine = 'Regressor' if self._dml.discrete_treatment is False else 'Classifier'
+        self.validate_score_types(score_types_dict, self.model_t_validation_score_activate, temp_engine, self._engine)
+        self.model_t_validation_prediction_type = self.update_validation_prediction_type(score_types_dict, temp_engine, self.model_t_validation_score_activate)
+        return score_types_dict
+
+    def get_model_t_validation_join_cols(self):
+        try:
+            join_cols = self._json_config["data"]["validation_params"]["model_t_scoring_params"]["join_cols"]
+        except KeyError:
+            join_cols = []
+        return join_cols
 
     def get_test_prediction_activate(self):
         try:
