@@ -3,7 +3,7 @@
 # Proprietary and confidential
 
 SUPPORTED_ENGINES = ["Recommender", "Classifier", "Regressor", "DeepClassifier", "DeepRegressor",
-                     "ImageClassifier"]
+                     "ImageClassifier", "DML"]
 
 SUPPORTED_MODELS = {"Classifier": ["AdaBoostClassifier",
                                    "BaggingClassifier",
@@ -24,7 +24,28 @@ SUPPORTED_MODELS = {"Classifier": ["AdaBoostClassifier",
                                    "QuadraticDiscriminantAnalysis",
                                    "RandomForestClassifier",
                                    "SVC",
-                                   "XGBClassifier"],
+                                   "XGBClassifier",
+                                   "CalibratedAdaBoostClassifier",
+                                   "CalibratedBaggingClassifier",
+                                   "CalibratedBernoulliNB",
+                                   "CalibratedCatBoostClassifier",
+                                   "CalibratedDecisionTreeClassifier",
+                                   "CalibratedExtraTreeClassifier",
+                                   "CalibratedExtraTreesClassifier",
+                                   "CalibratedGaussianNB",
+                                   "CalibratedGaussianProcessClassifier",
+                                   "CalibratedGradientBoostingClassifier",
+                                   "CalibratedKNeighborsClassifier",
+                                   "CalibratedLGBMClassifier",
+                                   "CalibratedLinearDiscriminantAnalysis",
+                                   "CalibratedLogisticRegression",
+                                   "CalibratedMLPClassifier",
+                                   "CalibratedMultinomialNB",
+                                   "CalibratedQuadraticDiscriminantAnalysis",
+                                   "CalibratedRandomForestClassifier",
+                                   "CalibratedSVC",
+                                   "CalibratedXGBClassifier",
+                                   ],
 
                     "Regressor": ["AdaBoostRegressor",
                                   "BaggingRegressor",
@@ -44,7 +65,8 @@ SUPPORTED_MODELS = {"Classifier": ["AdaBoostClassifier",
                                   "Ridge",
                                   "SGDRegressor",
                                   "SVR",
-                                  "XGBRegressor"],
+                                  "XGBRegressor"
+                                  ],
 
                     "DeepClassifier": ["DeepClassifier"],
 
@@ -80,7 +102,34 @@ SUPPORTED_MODELS = {"Classifier": ["AdaBoostClassifier",
                                         "Wide_ResNet101_2",
                                         "MNASNet0_5",
                                         "MNASNet1_0",
-                                        ]}
+                                        ],
+                    "DML": ["LinearDML",
+                            "DML",
+                            "NonParamDML"]
+                    }
+
+SUPPORTED_DML_MODEL_FINAL_TYPES = ["RidgeCV",
+                                   "LassoCV",
+                                   "AdaBoostRegressor",
+                                   "BaggingRegressor",
+                                   "CatBoostRegressor",
+                                   "DecisionTreeRegressor",
+                                   "ExtraTreeRegressor",
+                                   "ExtraTreesRegressor",
+                                   "GaussianProcessRegressor",
+                                   "GradientBoostingRegressor",
+                                   "HuberRegressor",
+                                   "KNeighborsRegressor",
+                                   "Lasso",
+                                   "LGBMRegressor",
+                                   "LinearRegression",
+                                   "RadiusNeighborsRegressor",
+                                   "RandomForestRegressor",
+                                   "Ridge",
+                                   "SGDRegressor",
+                                   "SVR",
+                                   "XGBRegressor"
+                                   ]
 
 classifier_scores = {"accuracy": "class",
                      "average_precision": "proba",
@@ -90,7 +139,7 @@ classifier_scores = {"accuracy": "class",
                      "f1": "class",
                      "fbeta": "class",
                      "hamming_loss": "class",
-                     "jaccard_similarity": "class",
+                     "jaccard": "class",
                      "log_loss": "proba",
                      "matthews_corrcoef": "class",
                      "precision": "class",
@@ -106,7 +155,8 @@ regressor_scores = {"explained_variance": None,
                     "mean_squared_log_error": None,
                     "median_absolute_error": None,
                     "r2": None,
-                    "root_mean_squared_error": None
+                    "root_mean_squared_error": None,
+                    "spearman_r": None
                     }
 
 deep_classifier_scores = {"accuracy": "class",
@@ -115,7 +165,7 @@ deep_classifier_scores = {"accuracy": "class",
                            "f1": "class",
                            "fbeta": "class",
                            "hamming_loss": "class",
-                           "jaccard_similarity": "class",
+                           "jaccard": "class",
                            "log_loss": "proba",
                            "matthews_corrcoef": "class",
                            "precision": "class",
@@ -129,7 +179,8 @@ SUPPORTED_SCORE_TYPES = {"Classifier": classifier_scores,
                          "Regressor": regressor_scores,
                          "DeepClassifier": deep_classifier_scores,
                          "DeepRegressor": regressor_scores,
-                         "ImageClassifier": deep_classifier_scores
+                         "ImageClassifier": deep_classifier_scores,
+                         "DML": {"MSE": None}
                          }
 
 SUPPORTED_INPUT_TYPES = {"Classifier": ["file", "query"],
@@ -137,7 +188,9 @@ SUPPORTED_INPUT_TYPES = {"Classifier": ["file", "query"],
                          "DeepClassifier": ["file", "query"],
                          "DeepRegressor": ["file", "query"],
                          "Recommender": ["file", "query"],
-                         "ImageClassifier": ["file", "folder"]}
+                         "ImageClassifier": ["file", "folder"],
+                         "DML": ["file", "query"]
+                         }
 
 SUPPORTED_SEGMENTERS = ["static", "dynamic", "value"]
 
@@ -167,6 +220,8 @@ SUPPORTED_MEAN_COMBINE_TYPES = ["arithmetic", "geometric"]
 
 SUPPORTED_ENGINE_FOR_PREDICTIONS_VS_ACTUALS_PLOT = ["Regressor", "DeepRegressor"]
 
+SUPPORTED_ENGINE_FOR_CALIBRATION_CURVE_PLOT = ["Classifier", "DeepClassifier"]
+
 SUPPORTED_STEMMERS = ["ARLSTem", "Cistem", "ISRIStemmer", "LancasterStemmer", "PorterStemmer", "RegexpStemmer",
                       "SnowballStemmer"]
 
@@ -175,3 +230,5 @@ SUPPORTED_GROUPBY_AGGREGATION_FUNCTIONS = ["mean", "sum", "size", "count", "std"
 SUPPORTED_ENGINES_FOR_LEARNING_RATE_PARAMS = ["DeepClassifier", "DeepRegressor", "ImageClassifier"]
 
 SUPPORTED_FILE_FORMATS = ["csv", "parquet", "feather"]
+
+SUPPORTED_VALIDATION_BEHAVIOR = ["error", "warning"]
