@@ -30,6 +30,7 @@ from nltk.stem.snowball import SnowballStemmer
 
 from learner.utilities.exclude import exclude
 from learner.configuration.configuration import Configuration
+from sklearn.decomposition import PCA
 
 
 class DataProcessor(object):
@@ -44,6 +45,8 @@ class DataProcessor(object):
         self._label_encoder = None
         self._learner_fill_nan = None
         self._standard_scaler = None
+        self._dml_standard_scaler = None
+        self._dml_pca = None
         self._min_max_scaler = None
         self._mean_dict = {}
         self._exclude_cols = exclude_cols if exclude_cols else []
@@ -90,6 +93,14 @@ class DataProcessor(object):
     def mean_dict(self, value):
         self._mean_dict = value
 
+    @property
+    def dml_standard_scaler(self):
+        return self._dml_standard_scaler
+
+    @property
+    def dml_pca(self):
+        return self._dml_pca
+
     @staticmethod
     def handle_copy_cols(data, cols=None):
         """Accept a dictionary with column names and their corresponding names for copying and make a copy of them.
@@ -510,8 +521,8 @@ class DataProcessor(object):
             from learner.feature_engineering.feature_engineering import FeatureEngineering
             data = FeatureEngineering.log_transform(data, params)
             # in case, we have NaN after the transformation, we use the DataValidator method to drop them
-            from learner.validator.data_validator import DataValidator
-            DataValidator.check_nulls_in_col(data, col=cols[0])
+            from learner.validator.data_validator import check_nulls_in_col
+            check_nulls_in_col(data, col=cols[0])
         return data
 
     @staticmethod
@@ -614,6 +625,84 @@ class DataProcessor(object):
         logging.info("Successfully processed the data")
         return self._conf, data
 
+    @exclude(name="cols")
+    def handle_dml_standard_scaler(self, data, cols=None, exclude_list=None):
+        """Accept a pandas dataframe and list of columns to standardized those columns by removing the mean and scaling
+        to unit variance, i.e. scaling the data so that the mean of the column is 0 and the standard deviation is 1.
+        If the dml_standard_scaler attribute is already populated, we use it to transform the data. This is usually the
+        case when using the test data. If dml_standard_scaler attribute is not populated, we first fit it to the data and
+        then transform it. This is usually the case when training the models. This method is identical to handle_standard_scaler,
+        but it populates different attributes that are only relevant to the dml engine. The logging message is also different.
+        It is more straightforward to re-implement a new method rather than updating the existing one.
+
+        :param data: a pandas dataframe
+        :param cols: a list of column to use for scaling
+        :param exclude_list: a list of items that should be excluded from cols (only the decorator function uses this)
+        :return: the updated data
+        """
+        if cols:
+            try:
+                if not self.dml_standard_scaler:
+                    # try to do the scaling in-place
+                    self._dml_standard_scaler = StandardScaler(copy=False)
+                    self._dml_standard_scaler.fit(data[cols])
+                    data[cols] = self._dml_standard_scaler.transform(data[cols])
+                else:
+                    data[cols] = self._dml_standard_scaler.transform(data[cols])
+            except Exception as e:
+                logging.critical(f"Unable to perform dml standard scaler for the selected columns. "
+                                 f"The error is {str(e)}. Exiting...")
+                sys.exit(1)
+        return data
+
+    @exclude(name="cols")
+    def handle_dml_pca(self, data, cols=None, exclude_list=None):
+        """This method is specific to the dml engine. Accept a pandas dataframe and list of columns to transform them using
+        pca. If the dml_pca attribute is already populated, we use it to transform the data. This is usually the
+        case when using the test data. If dml_pca attribute is not populated, we first fit it to the data and
+        then transform it. This is usually the case when training the models. After that, we add the new columns to the
+        dataframe and drop the old ones.
+
+        :param data: a pandas dataframe
+        :param cols: a list of column to use for scaling
+        :param exclude_list: a list of items that should be excluded from cols (only the decorator function uses this)
+        :return: the updated data
+        """
+        transformed = None
+        if cols:
+            try:
+                if not self.dml_pca:
+                    self._dml_pca = PCA(n_components=self.conf.dml.pca_n_components)
+                    self._dml_pca.fit(data[cols])
+                    transformed = self._dml_pca.transform(data[cols])
+                else:
+                    transformed = self._dml_pca.transform(data[cols])
+            except Exception as e:
+                logging.critical(f"Unable to perform dml PCA for the selected columns. The error is {str(e)}. "
+                                 f"Exiting...")
+                sys.exit(1)
+        pca_columns = [f'pca_{i + 1}' for i in range(transformed.shape[1])]
+        df_pca = pd.DataFrame(transformed, columns=pca_columns, index=data.index)
+        self.drop_cols(data, cols)
+        data = pd.concat([data, df_pca], axis=1)
+        return data
+
+    def dml_process_data(self, data, cols):
+        """This is the method that handles additional data processing for the dml engine. Depending on the user input,
+        it calls the appropriate methods to process the data.
+
+        :param data: a pandas dataframe
+        :param cols: a list of column to use for scaling
+        :return: the updated data
+        """
+        if self.conf.dml.standard_scaler_activate is True:
+            logging.info("Handling dml standard scaling... ")
+            data = self.handle_dml_standard_scaler(data, cols=cols)
+        if self.conf.dml.pca_activate is True:
+            logging.info("Handling dml pca... ")
+            data = self.handle_dml_pca(data, cols=cols)
+        return data
+
 
 class LearnerDummyEncoder:
     """LearnerDummyEncoder is a customized encoder that takes into consideration instances where the test data
@@ -1082,12 +1171,17 @@ def handle_date_cols(data, cols=None):
     :return: the updated dataframe with the new columns
     """
     for col, date_items in cols.items():
-        data[col] = pd.to_datetime(data[col],
-                                   errors='coerce',
-                                   infer_datetime_format=True)
+        try:
+            data[col] = pd.to_datetime(data[col],
+                                       errors='coerce',
+                                       infer_datetime_format=True)
+        # if we have Tz-aware datetime.datetime, we'll get value error. In that case, we set utc to true
+        except ValueError:
+            data[col] = pd.to_datetime(data[col],
+                                       errors='coerce',
+                                       infer_datetime_format=True,
+                                       utc=True)
         for item in date_items:
             data.loc[:, f"{col}_{item}"] = getattr(data[col].dt, item)
 
     return data
-
-
