@@ -39,6 +39,7 @@ class TrainDataManager:
         self.data = None
         self.processor = None
         self.feature_engineering = None
+        self.validator = None
         self.sample = None
 
     @property
@@ -103,9 +104,8 @@ class TrainDataManager:
 
         :return: None
         """
-        if self._conf.validation.validate:
-            validator = DataValidator(self._conf, self.data, data_type="train")
-            validator.validate_data()
+        self.validator = DataValidator(self._conf)
+        self.validator.validate_data(self.data, data_type="train")
 
     @timeit("analyze the training data")
     def analyze_data(self):
@@ -176,7 +176,7 @@ class ValidationDataManager:
     """This class manages the validation data for deep engines. The main method in this class is the `get_data_loader`
     method.  Unlike the class in data_manager module, we need to explicitly call that method to get the data_loader."""
 
-    def __init__(self, conf, mdl, processor=None, feature_engineering=None):
+    def __init__(self, conf, mdl, processor=None, feature_engineering=None, validator=None):
         """Initialize a ValidationDataManager object using a conf object, an item in models_dict, as well as the
         processor and feature_engineering objects.
 
@@ -189,6 +189,7 @@ class ValidationDataManager:
         self.data = None
         self.processor = processor
         self.feature_engineering = feature_engineering
+        self.validator = validator
 
     @property
     def conf(self):
@@ -250,9 +251,9 @@ class ValidationDataManager:
 
         :return: None
         """
-        if self._conf.validation.validate:
-            validator = DataValidator(self._conf, self.data, data_type="validation")
-            validator.validate_data()
+        if self.validator is None:
+            self.validator = DataValidator(self.conf)
+        self.validator.validate_data(self.data, data_type="validation")
 
     @timeit("process the validation data")
     def process_data(self):
@@ -304,7 +305,7 @@ class TestDataManager:
     # setting this so that pytest would not try to run this as a test
     __test__ = False
 
-    def __init__(self, conf, processor=None, feature_engineering=None):
+    def __init__(self, conf, processor=None, feature_engineering=None, validator=None):
         """Initialize a TestDataManager object using a conf object as well as the processor and feature_engineering
         objects.
 
@@ -315,6 +316,7 @@ class TestDataManager:
         self._conf = conf
         self.processor = processor
         self.feature_engineering = feature_engineering
+        self.validator = validator
         self.save_data()
 
     @property
@@ -386,9 +388,9 @@ class TestDataManager:
         :param data: a pandas dataframe
         :return: None
         """
-        if self._conf.validation.validate:
-            validator = DataValidator(self._conf, data, data_type="test")
-            validator.validate_data()
+        if self.validator is None:
+            self.validator = DataValidator(self.conf)
+        self.validator.validate_data(data, data_type="test")
 
     def process_data(self, data):
         """Process the data. Data processing is carried out based on user input in the config file.
