@@ -114,7 +114,7 @@ class Recommender(BaseRecommender):
             logging.info("Loading saved model...")
             self.load_models(self._conf.workspace.path)
 
-        if self._conf.validation.validate and self._conf.data.test_prediction_activate:
+        if self._conf.data.test_prediction_activate:
             self.validate_output()
 
     @timeit("train the recommendation model(s)")
@@ -144,7 +144,7 @@ class Recommender(BaseRecommender):
                             + str(self._conf.sep_timetag) + ".sav"
 
                 pickle_object(self.conf, self._conf.model.models_dict[tag]["path"], file_name, dict({tag: mdl}),
-                              self.processor, self.feature_engineering)
+                              self.processor, self.feature_engineering, self.validator)
 
             # handle feature importance if requested
             if self._conf.analysis.importance_activate:
@@ -197,7 +197,7 @@ class RecommenderSegment(BaseRecommender):
             output.concat_save_csv(self._conf.model.models_dict, self._conf.segmenter.seg_list,
                                    self._conf.workspace.name, self._conf.sep_timetag)
 
-        if self._conf.validation.validate and self._conf.data.test_prediction_activate:
+        if self._conf.data.test_prediction_activate:
             self.validate_output()
 
     @timeit("train the recommendation model(s)")
@@ -233,7 +233,7 @@ class RecommenderSegment(BaseRecommender):
                             + str(seg_id) + str(self._conf.sep_timetag) + ".sav"
 
                 pickle_object(self.conf, mdl["path"], file_name, dict({tag: mdl}),
-                              self.processor, self.feature_engineering)
+                              self.processor, self.feature_engineering, self.validator)
 
             # handle feature importance if requested
             if self._conf.analysis.importance_activate:
@@ -261,7 +261,7 @@ class RecommenderSegment(BaseRecommender):
 
         for path_to_pickle_dir, sample_pickle_file, prefix in zip(paths_to_pickle_dir, sample_pickle_files, prefixes):
 
-            self._conf, self.models_dict, self.processor, self.feature_engineering, sep_timetag = loader.load_model(sample_pickle_file)
+            self._conf, self.models_dict, self.processor, self.feature_engineering, self.validator, sep_timetag = loader.load_model(sample_pickle_file)
 
             for seg_id, seg_border in enumerate(self._conf.segmenter.seg_list):
                 logging.info("**************************************************************************")
@@ -273,7 +273,7 @@ class RecommenderSegment(BaseRecommender):
                     sep_timetag=str(sep_timetag),
                     ext=".sav"
                 )
-                self._conf, self.models_dict, self.processor, self.feature_engineering, sep_timetag = loader.load_model(filename)
+                self._conf, self.models_dict, self.processor, self.feature_engineering, self.validator, sep_timetag = loader.load_model(filename)
                 self.predict(seg_id, seg_border)
 
                 # we need to make sure we add the number of predictions from all the segments
@@ -307,7 +307,7 @@ class RecommenderSegment(BaseRecommender):
         :param seg_border: the upper boundary of the segment
         :return: None
         """
-        validation_manager = ValidationDataManager(self._conf, self.processor, self.feature_engineering)
+        validation_manager = ValidationDataManager(self._conf, self.processor, self.feature_engineering, self.validator)
         validation_data = self.filter_data(validation_manager.data, self._conf.segmenter.test_name, seg_border)
         for tag, mdls in self.models_dict.items():
             mdl = mdls.get(seg_id, mdls)
@@ -324,7 +324,7 @@ class RecommenderSegment(BaseRecommender):
         :param seg_border: the upper boundary of the seg
         :return: None
         """
-        test_manager = TestDataManager(self._conf, self.processor, self.feature_engineering)
+        test_manager = TestDataManager(self._conf, self.processor, self.feature_engineering, self.validator)
         reader, indices = test_manager.get_reader_for_segment(self._conf.segmenter.test_col, seg_border)
 
         num_chunks = 0
