@@ -18,6 +18,7 @@ class FeatureEngineering:
         self._conf = conf
         self._processor = None
         self._learner_groupby = None
+        self._learner_top_encoder = None
 
     @property
     def conf(self):
@@ -200,6 +201,22 @@ class FeatureEngineering:
             self._learner_groupby.transform(data)
         return data
 
+    def top_encoder(self, data, params):
+        """If learner_top_encoder is None (during training), instantiate a LearnerTopEncoder object and then call the
+        fit_transform method on it. During validation or testing, the learner_top_encoder would not be None. In that case,
+        we call the transform method on it.
+
+        :param data: the train or test dataframe
+        :param params: a list of top_encoder parameters coming from the conf object (see the configuration module for details)
+        :return: the updated dataframe
+        """
+        if not self._learner_top_encoder:
+            self._learner_top_encoder = LearnerTopEncoder(params)
+            self._learner_top_encoder.fit_transform(data)
+        else:
+            self._learner_top_encoder.transform(data)
+        return data
+
     def reprocess_data(self, data):
         """Once the feature engineering is completed, process the newly created columns to ensure the data is ready for
         training.
@@ -246,6 +263,10 @@ class FeatureEngineering:
             logging.info("Performing groupby feature engineering...")
             data = self.groupby(data, self._conf.feature_engineering.groupby_params)
 
+        if self._conf.feature_engineering.top_encoder_params:
+            logging.info("Performing top_encoder feature engineering...")
+            data = self.top_encoder(data, self._conf.feature_engineering.top_encoder_params)
+
         data = self.reprocess_data(data)
 
         return data
@@ -310,3 +331,72 @@ class LearnerGroupby:
         """
         self.fit(data)
         self.transform(data)
+
+
+class LearnerTopEncoder:
+    """LearnerTopEncoder is a class for performing the top encoding operations on the datasets. The top encoder
+    operation finds the categories with highest frequency in train data and encodes them into separate columns (1 if the
+    value equals to that category and 0 otherwise). This is useful when we have categorical columns with too many
+    categories in which dummy encoding is not really possible or useful. The reason for implementing a
+    custom class is to be able to use the values from the training data when working with validation or test datasets.
+    Basically, we do all the calculations during the training process. We then build a dictionary that maps the column
+    names to the values we'll be interested in, i.e. top n categories. When it comes to validation or test data, we'll
+    use that dictionary to created the new columns. During transformation, we'd need to create all the columns we
+    created for the training data even if none of the values exist in the validation or test data.
+    """
+    def __init__(self, params):
+        """Initialize an object of LearnerTopEncoder class using the parameters coming from the conf object. These
+        parameters have already been validated and updated. We do not perform any validation in this class.
+
+        :param params: a list of top_encoder parameters coming from the conf object (see the configuration module for details)
+        """
+        self._params = params
+        self._top_encoder_dict = dict()
+
+    @property
+    def top_encoder_dict(self):
+        return self._top_encoder_dict
+
+    def fit(self, data):
+        """If the params has values, loop through the parameters. Depending on whether we are doing "top_n" or
+        "min_portion", we call 'value_counts' method of pandas to get the categories and their counts. We then use that
+        data to populate top_encoder_dict for the column we are working on. Basically, for each column in the parameters
+        list, there will be a key in the top_encoder_dict. The value for each key will a list of categories we are
+        interested in. This dictionary will then be used to do the transformation.
+
+        :param data: a pandas dataframe. This is usually training dataset.
+        :return: None
+        """
+        if self._params:
+            for param in self._params:
+                if "top_n" in param:
+                    count_series = data[param["col"]].value_counts()
+                    self._top_encoder_dict[param["col"]] = list(count_series.index[:param["top_n"]])
+                elif "min_portion" in param:
+                    count_series = data[param["col"]].value_counts(normalize=True)
+                    self._top_encoder_dict[param["col"]] = list(count_series.index[count_series.ge(param["min_portion"])])
+
+    def transform(self, data):
+        """If the params has values, loop through the parameters and the list of values for the col in each parameter.
+        For each value, add a new column to the dataframe to encode that value.
+
+        :param data: a pandas dataframe. This is could be train, validation, or test data.
+        :return: None
+        """
+        for param in self._params:
+            col = param["col"]
+            for value in self._top_encoder_dict[col]:
+                col_name = f"{col}_top_encoder_{value}"
+                data[col_name] = data[col] == value
+                # convert true/false to 1/0 to not worry about processing it later
+                data[col_name] = data[col_name].astype(int)
+
+    def fit_transform(self, data):
+        """Call fit and transform method. This method is typically called during training while the transform method is
+        called during testing.
+
+        :param data: a pandas dataframe. This is could be train, validation, or test data.
+        :return: None
+        """
+        self.fit(data)
+        self.transform(data)
