@@ -18,6 +18,7 @@ from learner.model_manager.deep_classifiers import DeepClassifierHandler
 from learner.model_manager.deep_regressors import DeepRegressorHandler
 from learner.utilities.templates import PRED_FILENAME, PRED_FILENAME_SEG, PRED_FILENAME_CHUNK_SEG
 from learner.model_manager.image_classifiers import ImageClassifierHandler
+from learner.model_manager.dmls import DMLHandler
 
 
 class BasePredictor:
@@ -693,3 +694,30 @@ class ImageClassifierPredictor(BasePredictor):
             sys.exit(1)
         return prediction_df, data
 
+
+class DMLPredictor(BasePredictor):
+    """Handle making and writing predictions for the dml engine."""
+
+    def __init__(self, models_dict_item, conf):
+        """Initialize a DMLPredictor object by calling the parent initializer."""
+        super().__init__(models_dict_item, conf)
+
+    def make_predictions(self, test_data, column_name, data_type=None, prediction_type=None):
+        """Use the test_data to make predictions. Because the learner_predict returns both the effect and the interval,
+        the prediction dataframe will have three columns.
+
+        :param test_data: the data set to use for making predictions
+        :param column_name: the column name to use for writing the predictions
+        :param data_type: the data type, can be "validation" or "test"
+        :param prediction_type: the type of prediction for classification, can be "proba", "class", or "all". This parameter is not used here.
+        :return: predictions and data
+        """
+        effect, lower, upper, data = DMLHandler.learner_predict(test_data[self._models_dict_item["train_features"]],
+                                                                self._models_dict_item,
+                                                                carry_data=True,
+                                                                full_data=test_data,
+                                                                cols_to_carry=self._conf.column.id_cols)
+
+        prediction_df = pd.DataFrame(list(zip(effect, lower, upper)), columns=[column_name, f"{column_name}_lower", f"{column_name}_upper"])
+
+        return prediction_df, data
