@@ -12,12 +12,12 @@ import numpy as np
 import pandas as pd
 from sklearn.metrics import accuracy_score, f1_score, average_precision_score, precision_score, recall_score
 from sklearn.metrics import roc_auc_score, brier_score_loss, classification_report, confusion_matrix, fbeta_score
-from sklearn.metrics import hamming_loss, jaccard_similarity_score, log_loss, matthews_corrcoef, precision_recall_curve
+from sklearn.metrics import hamming_loss, jaccard_score, log_loss, matthews_corrcoef, precision_recall_curve
 from sklearn.metrics import precision_recall_fscore_support, zero_one_loss
 from sklearn.metrics import explained_variance_score, mean_absolute_error, mean_squared_error, mean_squared_log_error
 from sklearn.metrics import median_absolute_error, r2_score
 
-from learner.model_manager.scorers import root_mean_squared_error
+from learner.model_manager.scorers import root_mean_squared_error, spearman_r
 from learner.data_worker.data_loader import get_data
 from learner.configuration.supported_items import SUPPORTED_SCORE_TYPES
 from learner.configuration.configuration import Configuration
@@ -483,3 +483,100 @@ class DeepRegressorScorer(BaseScorer):
             model_score_dict[score_type] = self.scoring_handler(score_type, self._y_true, self._pred, params)
 
         return model_score_dict
+
+
+class DMLScorer(BaseScorer):
+    def __init__(self, conf: Configuration, models_dict, data, processor=None):
+        """Initialize a DMLScorer instance using a conf object and models_dict.
+
+        :param conf: configuration object
+        :param models_dict: learner models dictionary
+        """
+        super().__init__(conf, models_dict, processor)
+        self._t_true = self.get_t_true()
+        self._data = data
+
+    @property
+    def t_true(self):
+        return self._t_true
+
+    @property
+    def data(self):
+        return self._data
+
+    def score(self):
+        """The main function to compute the scores for the predictions generated by the dml model.
+
+        :return: None
+        """
+        for tag, mdls in self._models_dict.items():
+            if tag != "dml":
+                continue
+            mdl = mdls.get(0, mdls)
+            logging.info("Score the predictions for model %s", tag)
+
+            model_score_dict = self.get_model_score_dict()
+            for score_type, score in model_score_dict.items():
+                logging.info("%s model's %s score = %s", tag, score_type, score)
+
+    def get_model_score_dict(self):
+        """Make the call to the appropriate functions to calculate the score using the dml model. Unlike other scorer
+        classes, here we use the trained model object and call the "score" method to get the score, i.e. MSE.
+
+        :return: mode_score_dict that contains the score types and their corresponding values
+        """
+        model_score_dict = {}
+        for score_type, params in self._conf.data.validation_score_types.items():
+            model_score_dict[score_type] = self.models_dict["dml"]["model"].score(self.y_true, self.t_true, X=self.data)
+
+        return model_score_dict
+
+    def get_y_true(self):
+        """Load the model y target col from the validation data.
+
+        :return y_true: model y target col along with the join columns
+        """
+        logging.info("Loading model y target col from the validation data...")
+        y_true = get_data(self._conf.data.validation_location,
+                          manifest_file=self._conf.data.manifest,
+                          format=self._conf.data.validation_format,
+                          sep=self._conf.data.validation_delimiter,
+                          nrows=self._conf.data.validation_nrows,
+                          header=self._conf.data.validation_header,
+                          usecols=[self._conf.column.model_y_target_col] + self.conf.data.model_y_validation_join_cols)
+
+        # TODO: revisit the following two sections
+        # if we need to score the log transformed values, we do the transformation here
+        if self.conf.process.log_transform_target_activate and not self.conf.process.log_transform_target_score_actuals:
+            from learner.data_worker.data_processor import log_transform
+            y_true = log_transform(data=y_true, params=self._conf.process.log_transform_target_params, cols=[self._conf.column.model_y_target_col])
+        # in case target column was label encoded, use the trained label encoder in the processor object to
+        # label encode y_true
+        if self._processor and self._conf.process.label_encode_target:
+            self._processor.handle_label_encoding(data=y_true, cols=[self._conf.column.model_y_target_col])
+        return y_true
+
+    def get_t_true(self):
+        """Load the model t target col from the validation data.
+
+        :return t_true: model t target col along with the join columns
+        """
+        logging.info("Loading model t target col from the validation data...")
+        y_true = get_data(self._conf.data.validation_location,
+                          manifest_file=self._conf.data.manifest,
+                          format=self._conf.data.validation_format,
+                          sep=self._conf.data.validation_delimiter,
+                          nrows=self._conf.data.validation_nrows,
+                          header=self._conf.data.validation_header,
+                          usecols=[self._conf.column.model_t_target_col] + self.conf.data.model_t_validation_join_cols)
+
+        # TODO: revisit the following two sections
+        # if we need to score the log transformed values, we do the transformation here
+        if self.conf.process.log_transform_target_activate and not self.conf.process.log_transform_target_score_actuals:
+            from learner.data_worker.data_processor import log_transform
+            y_true = log_transform(data=y_true, params=self._conf.process.log_transform_target_params, cols=[self._conf.column.model_t_target_col])
+        # in case target column was label encoded, use the trained label encoder in the processor object to
+        # label encode y_true
+        if self._processor and self._conf.process.label_encode_target:
+            self._processor.handle_label_encoding(data=y_true, cols=[self._conf.column.model_t_target_col])
+        return y_true
